package com.zanxiang.sdk.service.pay;

import com.alibaba.fastjson.JSONObject;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.sdk.constant.MiPayConstants;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.GameAppletDTO;
import com.zanxiang.sdk.domain.dto.MiPayConfigDTO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.service.GameAppletService;
import com.zanxiang.sdk.service.OrderService;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.util.WxPayUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 米大师服务实现
 *
 * @author xufeng
 * @date 2022/7/14 16:33
 */
@Slf4j
@Service
public class MiPayService extends PayBaseService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private GameAppletService gameAppletService;

    @Value("${spring.profiles.active}")
    private String springActive;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private UserService userService;

    /**
     * 米大师支付配置
     */
    private MiPayConfigDTO config;

    private int isSand = 1;

    @Override
    public ResultMap create(ProductPayParamBO product) {
        //查询订单
        PlatformOrderDTO orderInfo = orderService.info(product.getOutTradeNo());
        if (Objects.isNull(orderInfo)) {
            throw new CustomException(HttpStatusEnum.ORDER_NO_FIND);
        }
        if (orderInfo.getGameId() == null || orderInfo.getGameId() == 0) {
            throw new CustomException(HttpStatusEnum.ORDER_GAME_ID_IS_NULL);
        }
        this.isSand = Objects.equals(this.springActive, "dev") ? 1 : 0;
        this.configInit(product.getGameId());
        //获取用户openId
        UserDTO userDTO = userService.getUserByUserId(product.getUserId());
        String openId = userDTO.getOpenId();
        //获取米大师钱包余额
        Long balance = this.midasGetBalance(openId);
        //返回参数
        Map<String, String> result = new HashMap<>();
        //余额充足, 直接扣除
        if (balance >= orderInfo.getAmount().longValue()) {
            String billNo = this.midasPay(openId, orderInfo);
            result.put("status", "0");
            result.put("billNo", billNo);
        }
        //余额不足, 返回订单id, 前端调充值接口, 充值完后再继续支付
        if (balance < orderInfo.getAmount().longValue()) {
            result.put("currencyType", "CNY");
            result.put("mode", "game");
            result.put("zoneId", "1");
            result.put("offerId", config.getAppId());
            result.put("env", String.valueOf(this.isSand));
            result.put("buyQuantity", orderInfo.getAmount().toString());
            result.put("status", "1");
            result.put("balance", String.valueOf(balance));
            result.put("orderId", product.getOutTradeNo());
        }
        return ResultMap.ok(result);
    }

    private Long midasGetBalance(String openId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("openid", openId);
        paramMap.put("appid", config.getWxAppId());
        paramMap.put("offer_id", config.getAppId());
        paramMap.put("ts", String.valueOf(System.currentTimeMillis() / 1000));
        paramMap.put("zone_id", "1");
        paramMap.put("pf", "android");



        String urlPath = this.isSand == 1 ? "/cgi-bin/midas/sandbox/getbalance" : "/cgi-bin/midas/getbalance";
        String appKey = this.isSand == 1 ? config.getAppKeyDev() : config.getAppKey();
        String sig = WxPayUtil.miPaySin("POST", urlPath, paramMap, appKey);



        paramMap.put("sig", this.sign("POST", paramMap));
        Map<String, String> result = this.miPayApi(MiPayConstants.BALANCE_URL, paramMap);
        return Long.valueOf(result.get("balance"));
    }

    private String midasPay(String openId, PlatformOrderDTO orderInfo) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("appid", config.getWxAppId());
        paramMap.put("openid", openId);
        paramMap.put("offer_id", config.getAppId());
        paramMap.put("ts", String.valueOf(System.currentTimeMillis() / 1000));
        paramMap.put("pf", "android");
        paramMap.put("zone_id", "1");
        paramMap.put("amt", orderInfo.getAmount().toString());
        paramMap.put("bill_no", orderInfo.getOrderId());
        paramMap.put("sig", this.sign("POST", paramMap));
        Map<String, String> result = this.miPayApi(MiPayConstants.PAY_URL, paramMap);
        return result.get("bill_no");
    }

    private String getAccessToken(String appId, String appSecret) {
//        String redisKey = RedisKeyConstant.WEIXIN_ACCESS_TOKEN + appId;
//        String cache = redisUtil.getCache(redisKey);
//        if (StringUtils.isNotEmpty(cache)) {
//            return cache;
//        }
        HashMap<String, String> param = new HashMap<>();
        param.put("grant_type", "client_credential");
        param.put("appid", "wxa8ede4e6c65fad30");
        param.put("secret", "a8df1c1146f89f06d07d1e494f845ede");

        String url = URIUtil.fillUrlParams(MiPayConstants.TOKEN_URL, param, Boolean.FALSE);

        String result = restTemplate.getForObject(url, String.class);


        if (StringUtils.isEmpty(result)) {
            return "";
        }
        JSONObject data = JSONObject.parseObject(result);
        if (Objects.isNull(data)) {
            return "";
        }
        Object accessToken = data.get("access_token");
        if (Objects.isNull(accessToken)) {
            return "";
        }
        String token = accessToken.toString();
//        redisUtil.setCache(redisKey, token, Long.valueOf(data.get("expires_in").toString()));
        return token;
    }

    private Map<String, String> miPayApi(String url, Map<String, String> paramMap) {
        try {
            //使用沙箱时，沙箱地址替换
            String appKey = config.getAppKey();
            if (this.isSand == 1) {
                url = url.replace("/cgi-bin/midas/", "/cgi-bin/midas/sandbox/");
                appKey = config.getAppKeyDev();
            }
            String accessToken = getAccessToken(config.getAppId(), appKey);
            if (StringUtils.isEmpty(accessToken)) {
                throw new CustomException(HttpStatusEnum.ACCESS_TOKEN_CREATE_ERROR);
            }

            url += "?access_token=" + accessToken;
            String result = restTemplate.postForObject(url, paramMap, String.class);
            if (StringUtils.isEmpty(result)) {
                throw new CustomException(HttpStatusEnum.MDS_CONNECT_ERROR);
            }
            Map<String, String> data = JsonUtil.toMap(result, Map.class, String.class, String.class);
            if (Objects.isNull(data)) {
                throw new CustomException(HttpStatusEnum.MDS_CONNECT_ERROR);
            }
            if (!data.get("errcode").equals("0")) {
                String errmsg = data.get("errmsg");
                Integer errcode = Integer.valueOf(data.get("errcode"));
                throw new CustomException(errmsg, errcode);
            }
            return data;
        } catch (Exception e) {
            throw new BaseException("米大师接口调用失败");
        }
    }

    private void configInit(Long gameId) {
        //查询游戏小程序信息
        GameAppletDTO gameAppletDTO = gameAppletService.getByGameId(gameId);
        //支付配置赋值
        this.config = MiPayConfigDTO.builder()
                .wxAppId(gameAppletDTO.getAppId())
                .appId(gameAppletDTO.getMiPayAppId())
                .appKey(gameAppletDTO.getMiPayAppKey())
                .appKeyDev(gameAppletDTO.getMiPayAppKeyDev())
                .build();
    }

    private String sign(String method, Map<String, String> paramMap) {
        String urlPath = this.isSand == 1 ? "/cgi-bin/midas/sandbox/getbalance" : "/cgi-bin/midas/getbalance";
        String appKey = this.isSand == 1 ? config.getAppKeyDev() : config.getAppKey();
        return WxPayUtil.miPaySin(method, urlPath, paramMap, appKey);
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        throw new BaseException("米大师支付不存在异步回调");
    }

    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        throw new BaseException("米大师支付不存在同步回调");
    }
}
